'use strict';

/* istanbul ignore file: ignore from coverage because it varies per eslint version */

var eslintNaming = void 0;

function _getNormalizer() {
  if (eslintNaming) {
    return eslintNaming;
  }

  var eslintVersionFunctions = [
  // eslint >= 6.1.0
  function () {
    var normalizer = require('eslint/lib/shared/naming');

    return {
      normalizePackageName: normalizer.normalizePackageName,
      getShorthandName: normalizer.getShorthandName
    };
  },
  // eslint 6.0.0 - 6.0.1
  function () {
    var normalizer = require('eslint/lib/cli-engine/naming');

    return {
      normalizePackageName: normalizer.normalizePackageName,
      getShorthandName: normalizer.getShorthandName
    };
  },
  // eslint 5
  function () {
    var normalizer = require('eslint/lib/util/naming');

    return {
      normalizePackageName: normalizer.normalizePackageName,
      getShorthandName: normalizer.getShorthandName
    };
  },
  // eslint 4
  function () {
    var normalizer = require('eslint/lib/util/naming');

    return {
      normalizePackageName: normalizer.normalizePackageName,
      getShorthandName: normalizer.removeNamespaceFromTerm
    };
  },
  // eslint 3
  function () {
    var normalizer = require('eslint/lib/config/plugins');

    var PLUGIN_NAME_PREFIX = 'eslint-plugin-';

    function parsePluginName(pluginName) {
      var pluginNamespace = normalizer.getNamespace(pluginName);
      var pluginNameWithoutNamespace = normalizer.removeNamespace(pluginName);
      var pluginNameWithoutPrefix = normalizer.removePrefix(pluginNameWithoutNamespace);

      return {
        pluginNamespace: pluginNamespace,
        pluginNameWithoutPrefix: pluginNameWithoutPrefix
      };
    }

    function normalizePackageName(pluginName) {
      var sections = parsePluginName(pluginName);
      var longName = sections.pluginNamespace + PLUGIN_NAME_PREFIX + sections.pluginNameWithoutPrefix;

      return longName;
    }

    function getShorthandName(pluginName) {
      var sections = parsePluginName(pluginName);
      var shortName = sections.pluginNamespace + sections.pluginNameWithoutPrefix;

      return shortName;
    }

    return {
      normalizePackageName: normalizePackageName,
      getShorthandName: getShorthandName
    };
  }];

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = eslintVersionFunctions[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var tryEslintVersion = _step.value;

      try {
        var normalizer = tryEslintVersion();

        if (typeof normalizer.normalizePackageName === 'function' && typeof normalizer.getShorthandName === 'function') {
          eslintNaming = {
            normalizePackageName: normalizer.normalizePackageName,
            getShorthandName: normalizer.getShorthandName
          };

          return eslintNaming;
        }
      } catch (err) {}
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  throw new Error('eslint naming functions not found');
}

function normalizePluginName(name) {
  var normalizer = _getNormalizer();

  var module = normalizer.normalizePackageName(name, 'eslint-plugin');
  var prefix = normalizer.getShorthandName(name, 'eslint-plugin');

  return { module: module, prefix: prefix };
}

module.exports = normalizePluginName;